@@ -77,6 +77,9 @@ module POSIX
       #   :max     => total    Maximum number of bytes of output to allow the
       #                        process to generate before aborting with a
       #                        MaximumOutputExceeded exception.
+      #   :pgroup_kill => bool Boolean specifying whether to kill the process
+      #                        group (true) or individual process (false, default).
+      #                        Setting this option true implies :pgroup => true.
       #
       # Returns a new Child instance whose underlying process has already
       # executed to completion. The out, err, and status attributes are
@@ -87,6 +90,10 @@ module POSIX
         @input = @options.delete(:input)
         @timeout = @options.delete(:timeout)
         @max = @options.delete(:max)
+        if @options.delete(:pgroup_kill)
+          @pgroup_kill = true
+          @options[:pgroup] = true
+        end
         @options.delete(:chdir) if @options[:chdir].nil?
         exec! if !@options.delete(:noexec)
       end
@@ -113,7 +120,7 @@ module POSIX
           else
             {}
           end
-        new(*args, { :noexec => true }.merge(options))
+        new(*(args + [{ :noexec => true }.merge(options)]))
       end
 
       # All data written to the child process's stdout stream as a String.
@@ -128,6 +135,11 @@ module POSIX
       # Total command execution time (wall-clock time)
       attr_reader :runtime
 
+      # The pid of the spawned child process. This is unlikely to be a valid
+      # current pid since Child#exec! doesn't return until the process finishes
+      # and is reaped.
+      attr_reader :pid
+
       # Determine if the process did exit with a zero exit status.
       def success?
         @status && @status.success?
@@ -139,17 +151,22 @@ module POSIX
       def exec!
         # spawn the process and hook up the pipes
         pid, stdin, stdout, stderr = popen4(@env, *(@argv + [@options]))
+        @pid = pid
 
         # async read from all streams into buffers
         read_and_write(@input, stdin, stdout, stderr, @timeout, @max)
 
         # grab exit status
         @status = waitpid(pid)
-      rescue Object => boom
+      rescue Object
         [stdin, stdout, stderr].each { |fd| fd.close rescue nil }
         if @status.nil?
-          ::Process.kill('TERM', pid) rescue nil
-          @status = waitpid(pid)      rescue nil
+          if !@pgroup_kill
+            ::Process.kill('TERM', pid) rescue nil
+          else
+            ::Process.kill('-TERM', pid) rescue nil
+          end
+          @status = waitpid(pid) rescue nil
         end
         raise
       ensure
@@ -180,7 +197,6 @@ module POSIX
       def read_and_write(input, stdin, stdout, stderr, timeout=nil, max=nil)
         max = nil if max && max <= 0
         @out, @err = '', ''
-        offset = 0
 
         # force all string and IO encodings to BINARY under 1.9 for now
         if @out.respond_to?(:force_encoding) and stdin.respond_to?(:set_encoding)
